##############################################################################
# $Id: process.pl,v 1.2 1999/08/26 04:06:57 jheiss Exp $
##############################################################################
# Subroutines which check on or stop processes on remote hosts.
##############################################################################
# Copyright (C) 1999  Jason Heiss (jheiss@ofb.net)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##############################################################################
# Pre-RCS version control:
# created 09/15/97
# last modified 04/13/98
##############################################################################
# $Log: process.pl,v $
# Revision 1.2  1999/08/26 04:06:57  jheiss
# Added copyright and GNU license message.
#
# Revision 1.1  1999/08/09 03:06:55  jheiss
# Initial revision
#
##############################################################################

require 'info.pl';
require 'rsh.pl';

sub _pscmd
{
	my $host = $_[0];
	my $pscmd;

	my $os = get_os($host);
	my $osmajor = get_os_major_version($host);
 
	if ($os eq 'SunOS')
	{
		if ($osmajor == 4)
		{
			$pscmd = 'ps -aux';
		}
		elsif ($osmajor == 5)
		{
			$pscmd = 'ps -ef';
		}
		else
		{
			die "Unknown major version of SunOS ($osmajor) on $host\n";
		}
	}
	elsif ($os eq 'HP-UX')
	{
		$pscmd = 'ps -ef';
	}
	else
	{
		die "Unknown OS ($os) on $host";
	}

	return $pscmd;
}

sub process_check
{
	my $host = $_[0];
	my $process = $_[1];
	my $retline = $_[2];
	my $pscmd = _pscmd($host);
 
	unless (defined($host) && defined($process) && defined($pscmd))
	{
		die "Invalid arguments to \&process_check";
	}

	open(PS, "rsh $host $pscmd|");
	while(<PS>)
	{
		if (/$process/)
		{
			close(PS);
			if ($retline)
			{
				return $_;
			}
			else
			{
				return 1;
			}
		}
	}
	close(PS);
        
	return 0;
}

sub process_kill
{
	my $host = $_[0];
	my $name = $_[1];
	my $dohup = $_[2];
	my $pscmd = _pscmd($host);

	unless (defined($host) && defined($name) && defined($pscmd))
	{
		die "Invalid arguments to \&process_kill";
	}
 
	open(PS, "rsh $host $pscmd|");
	while(<PS>)
	{
		if (/$name/)
		{
			my ($owner, $pid) = split;
			if ($dohup)
			{
				$hup = '-HUP';
			}
			else
			{
				$hup = '';
			}

			rsh($host, "kill $hup $pid 2>&1 | grep -v 'no such process'");
		}
	}
	close(PS);
}

sub process_killhup
{
	process_kill($_[0], $_[1], 1);
}

return 1;

