##############################################################################
# $Id: rsh.pl,v 1.3 1999/08/26 04:06:57 jheiss Exp $
##############################################################################
# Subroutines for executing commands on remote machines.
##############################################################################
# Copyright (C) 1999  Jason Heiss (jheiss@ofb.net)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##############################################################################
# Pre-RCS version control:
# created 10/01/97
# last modified 04/13/98
##############################################################################
# $Log: rsh.pl,v $
# Revision 1.3  1999/08/26 04:06:57  jheiss
# Added copyright and GNU license message.
#
# Revision 1.2  1999/08/09 04:00:20  jheiss
# Fixed a typo
#
# Revision 1.1  1999/08/09 03:12:48  jheiss
# Initial revision
#
##############################################################################

#############
# Constants #
#############
$RETURNERRS = 1;
$NORETURNERRS = 0;
$SPLIT = 0;
$NOSPLIT = 1;
$RSH='/usr/bin/rsh';
#$RSH='/usr/local/bin/ssh';

# Our generic rsh subroutine which actually does the rsh.  The various
# specialized subroutines later in this file call this subroutines with
# various arguments which tell it how they want things handled.
sub _rsh
{
	my $host = $_[0];
	my $cmd = $_[1];
	my $return_errors = $_[2];
	my $no_split = $_[3];
	my ($cmd1, $cmd2);

	# This subroutine should only be called by other subroutines in this
	# file.  Since they already check their arguments for validity we don't
	# need to waste time repeating that here.

	$return_errors = 0 unless (defined($return_errors));
	$no_split = 0 unless (defined($no_split));

	my $errcode;
	if ($return_errors)
	{
		$errcode = '2>&1';
	}
	else
	{
		$errcode = '2>/dev/null';
	}

	# Normally we want to insert the $errcode chunk between the first
	# and second commands in a line of pipes.  This insures that stderr
	# from the first command is either removed or redirected to stdout
	# before the output is passed to other programs.  However, in certain
	# rare circumstances we don't want to do that.
	if ($no_split)
	{
		$cmd1 = $cmd;   # Everything goes in to first command
		$cmd2 = 'cat';  # I.e. don't do anything
	}
	else
	{
		($cmd1, $cmd2) = split(/\|/, $cmd, 2);
		$cmd2 = 'cat' unless (defined($cmd2));  # If no pipes in command
	}

	# Fun with shell quoting rules...
	my $command = "$RSH $host \"sh -c \\\"$cmd1 $errcode | " .
		"$cmd2\\\"\" | grep -v 'Application Server not found'";

	#print "_rsh sees cmd1 as $cmd1\n";
	#print "_rsh sees cmd2 as $cmd2\n";
	#print "_rsh executing:\n    $command\n";

	# Execute the command
	my $output = `$command`;

	#print "output is $output\n";

	return $output;
}

################################################################
# Return the results (stdout and stderr combined) of a command #
################################################################

sub rsh
{
	my $host = $_[0];
	my $cmd = $_[1];
 
	unless (defined($host) && defined($cmd))
	{
		die "Invalid arguments to \&rsh";
	}

	return _rsh($host, $cmd, $RETURNERRS, $SPLIT);
}
 
########################################################
# Return the results of a command but strip out stderr #
########################################################

sub rsh_noerr
{
	my $host = $_[0];
	my $cmd = $_[1];
 
	unless (defined($host) && defined($cmd))
	{
		die "Invalid arguments to \&rsh_noerr";
	}

	return _rsh($host, $cmd, $NORETURNERRS, $SPLIT);
}

###########################################################################
# _rsh normally splits the given command on the first pipe and inserts    #
# some stuff to strip out error messages.  Sometimes this isn't desirable #
# so we provide a function that doesn't do that.                          #
###########################################################################

sub rsh_nosplit
{
	my $host = $_[0];
	my $cmd = $_[1];
 
	unless (defined($host) && defined($cmd))
	{
		die "Invalid arguments to \&rsh_nosplit";
	}

	return _rsh($host, $cmd, $RETURNERRS, $NOSPLIT);
}

############################################
# Return just the exit code from a command #
############################################

sub rsh_return
{
	my $host = $_[0];
	my $cmd = $_[1];
 
	unless (defined($host) && defined($cmd))
	{
		die "Invalid arguments to \&rsh_return";
	}

	# This is a little tricky...  The return value from "rsh host command" is
	# the return value of the rsh, not the command.  It indicates whether the
	# rsh itself was successful.  So we sneak an echo of the exit code into
	# the command that is passed to rsh.

	# Even more fun with shell quoting rules
	my $command = "rsh $host \"sh -c '$cmd > /dev/null 2>&1 ; echo \\\$?'\" |" .
		" grep -v 'Application Server not found'";

	chop(my $output = `$command`);

	return $output;
}

#####################################################
# Check to see if a file exists on a remote machine #
#####################################################
sub rsh_file_exists
{
	my $host = $_[0];
	my $file = $_[1];
 
	unless (defined($host) && defined($file))
	{
		die "Invalid arguments to \&rsh_file_exists";
	}

	#print "\nChecking for: $file on $host\n";
	#print "We got back: " . _rsh($host, "ls $file", 1) . "\n";

	# SunOS 4.x will say 'not found'
	# SunOS 5.x will say 'No such file or directory'
	# HP-UX will say 'not found'
	my $returned_text = _rsh($host, "ls $file", 1);
	if ($returned_text =~ /No such file or directory/ ||
		$returned_text =~ /not found/)
	{
		#print "rsh_file_exists returning 0\n";
		return 0;
	}
	else
	{
		#print "rsh_file_exists returning 1\n";
		return 1;
	}
}

#########################################
# rcp a local file to a remote location #
#########################################
sub rcp
{
	my $host = $_[0];
	my $localfile = $_[1];
	my $remotefile = $_[2];

	unless (defined($host) && defined($localfile) && defined($remotefile))
	{
		die "Invalid arguments to \&rcp";
	}

	system("rcp $localfile $host:$remotefile");
}

return 1;

