##############################################################################
# $Id: sort.pl,v 1.2 1999/08/26 04:06:57 jheiss Exp $
##############################################################################
# Various useful subroutines for sort.
##############################################################################
# Copyright (C) 1999  Jason Heiss (jheiss@ofb.net)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##############################################################################
# Pre-RCS version control:
# created 03/20/98
# last modified 03/20/98
##############################################################################
# $Log: sort.pl,v $
# Revision 1.2  1999/08/26 04:06:57  jheiss
# Added copyright and GNU license message.
#
# Revision 1.1  1999/08/09 03:14:11  jheiss
# Initial revision
#
##############################################################################

sub byhostname
{
	# HSC hostnames are generally of the form:
	# machine type + number + optional suffix
	# I.e. hscw1149 or hscs022-f0
	# We want to sort these so that hscw1149 appears after hscw666.  If
	# we just sorted them alphabetically that wouldn't happen.  So we
	# split the name into the three parts, sort machine type alphabetically
	# then number numerically and finally the suffix alphabetically.
	my ($atype, $anumber, $arest) = split(/(\d+)/, $a, 3);
	my ($btype, $bnumber, $brest) = split(/(\d+)/, $b, 3);

	$atype cmp $btype
		or
	$anumber <=> $bnumber
		or
	$arest cmp $brest
		or
	$a cmp $b;
}

# This occasionally causes a seg fault because it isn't well ordered
sub random
{
	if (defined($randcache{"$a:$b"}))
	{
		print "Using cached value\n";
	}
	else
	{
		my $rand = rand;

		if ($rand > .5)
		{
			$randcache{"$a:$b"} = 1;
			return 1;
		}
		else
		{
			$randcache{"$a:$b"} = -1;
			return -1;
		}
	}
}

sub sort_randomly
{
	my %randoms;
	my @sorted;

	foreach $value (@_)
	{
		my $rand;

		do
		{
			$rand = rand;
		}
		while (defined($randoms{$rand}));

		$randoms{$rand} = $value;
	}

	foreach $randkey (sort keys %randoms)
	{
		push(@sorted, $randoms{$randkey});
	}

	return @sorted;
}

return 1;

