#!/usr/bin/perl -T
##############################################################################
# $Id$
##############################################################################
# This CGI script allows a user to check the status of Jumpstarts performed
# using the scripts developed by Jason Heiss.
# It does so by reading the status file that those scripts keep updated and
# generating HTML pages.
# 
# Note:  In order for this script to be able to read that status file it
# expects that /export/install and /export/logs will be mounted from hscj002.
# I've set this up in /etc/auto.direct on taggart.es.hac.com.  If for some
# reason this gets moved to another server, something similar will need to be
# done there.
##############################################################################
# Copyright (C) 1999  Jason Heiss (jheiss@ofb.net)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##############################################################################
# Pre-RCS version control:
# created 7/22/97
# last modified 05/01/98
##############################################################################
# $Log$
##############################################################################

#############
# Libraries #
#############

push(@INC, '/export/install/Scripts');
require 'inarray.pl';
require 'basedir.pl';
require 'sort.pl';
require 'status.pl';
push(@INC, '/home1/isdg/perl');
require 'cgi-lib.pl';

#############
# Constants #
#############
my $UPDATE=120;  # Measured in seconds

my $SPECIAL_EVENT_DIR = '/export/install/Scripts/hostlists';
my %SPECIAL_EVENTS = (
	'mo', 'Model Office',
	'trainroom', 'Solaris Training Room',
	'21pilot', '2.1 Solaris Standard Client Pilot',
	'21clients', '2.1 Solaris Standard Client Rollout',
	'21clients.makeup', '2.1 Solaris Standard Client Rollout to E01/Misc.',
	'21clients.s12', '2.1 Solaris Standard Client Rollout to S12',
	'21clients.s50and21', '2.1 Solaris Standard Client Rollout to S50/S21',
	'21clients.s24and25', '2.1 Solaris Standard Client Rollout to S24/S25',
	'21clients.cleanup', '2.1 Solaris Standard Client Rollout cleanup',
);

########################
# Print the CGI header #
########################
print PrintHeader();

############################
# Read in the user's input #
############################
$rp = ReadParse();

########################################################################
# If the status file is locked we inform the user and redirect them to #
# a page that waits for the lock to be released.                       #
########################################################################
#if (!$in{redirect} && isstatlocked())
if (0)
{
	print <<EOF;
<HTML>
<HEAD>
<META HTTP-EQUIV=\"Refresh\" CONTENT=\"0; URL=status.cgi?$ENV{QUERY_STRING}&redirect=1\">
</HEAD>
<BODY>

<CENTER>
<H1>The status file is locked</H1>
</CENTER>

<HR>

<P>
Please wait while we get a lock on the status file...
</P>

<P>
If this times out, <A HREF="status.cgi">try again</A>.
</P>

</BODY>
</HTML>
EOF
	exit;
}

#########################################
# Read in the info from the status file #
#########################################

# The second argument to readstat tells it to print warnings as HTML comments.
# Keeps the resulting HTML page cleaner.  The third argument tells the
# status stuff that we are ok with no superlocking first.
readstat(0, 1, 100);

# If the user specified a special event then load that file of hosts
# to view, otherwise just put all hosts in the statusfile in the list
# of hosts to view.
if (defined($in{event}) && inarray($in{event}, keys %SPECIAL_EVENTS))
{
	@hosts_to_view = readinputfile("$SPECIAL_EVENT_DIR/$in{event}");
	$eventurl = "&event=$in{event}";
}
else
{
	@hosts_to_view = @hosts;
	$eventurl = '';
}

# Store a sorted lists of the hosts
@shosts = sort byhostname @hosts_to_view;

# Build a list of all of the buildings that the hosts are in
@builds = getbuilds(@hosts_to_view);

########################################################
# Grab the time so we can put it in the generated HTML #
########################################################
my $now = localtime(time);

#######################################################################
# Figure out when the status file was last updated so we can put that #
# on the generated page also.                                         #
#######################################################################
my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime,
		$mtime, $ctime, $blksize, $blocks) = stat($STATFILE);
my $statmtime = localtime($mtime);

############################################################################
# If the user hasn't selected a building to view we output the intro page. #
############################################################################
unless ($rp && defined($in{build}))
{
	print <<EOF;
<HTML>
<HEAD>
<META HTTP-EQUIV="Refresh" CONTENT=$UPDATE>
<TITLE>Unix Infrastructure Jumpstart Status Page</TITLE>
</HEAD>
<BODY>

<CENTER>
<H1>Welcome to the Unix Infrastructure Jumpstart Status Page</H1>
<EM>This page generated at $now</EM><BR>
<EM>Status file last modified at $statmtime</EM>
</CENTER>

<HR>

Special events:
<UL>
EOF

	foreach $event (sort keys %SPECIAL_EVENTS)
	{
		print "<LI><A HREF=\"status.cgi?event=$event\">" .
			"$SPECIAL_EVENTS{$event}</A></LI>\n";
	}

	print <<EOF;
</UL>

<HR>
EOF

	if ($in{event})
	{
		print <<EOF;
<STRONG>Special event "$SPECIAL_EVENTS{$in{event}}" selected</STRONG>

<HR>
EOF
	}

print <<EOF;
All machines:
<UL>
<LI><A HREF="status.cgi?build=all&sort=host$eventurl">All machines
    (sorted by hostname)</A></LI>
<LI><A HREF="status.cgi?build=all&sort=loc$eventurl">All machines
    (sorted by location)</A></LI>
<LI><A HREF="status.cgi?build=all&sort=stat$eventurl">All machines
    (sorted by status)</A></LI>
</UL>
EOF

	# By building, sorted by hostname
	print "By building (sorted by hostname):\n<UL>\n";
	foreach $build (@builds)
	{
		print "<LI><A HREF=\"status.cgi?build=$build&sort=host$eventurl\">" .
			"$build machines</A></LI>\n";
	}
	print "</UL>\n";
	
	# By building, sorted by location
	print "By building (sorted by location):\n<UL>\n";
	foreach $build (@builds)
	{
		print "<LI><A HREF=\"status.cgi?build=$build&sort=loc$eventurl\">" .
			"$build machines</A></LI>\n";
	}
	print "</UL>\n";

	# By building, sorted by status
	print "By building (sorted by status then location):\n<UL>\n";
	foreach $build (@builds)
	{
		print "<LI><A HREF=\"status.cgi?build=$build&sort=stat$eventurl\">" .
			"$build machines</A></LI>\n";
	}
	print "</UL>\n";

	print <<EOF;
<HR>

<ADDRESS>jheiss\@hsc.hac.com</ADDRESS>

</BODY>
</HTML>
EOF
}

#####################################################
# User selected a building, show him those machines #
#####################################################
else
{
	%sorttext = ('host', 'hostname', 'loc', 'location', 'stat', 'status');
	%sortsub = ('host', \&byhostname, 'loc', \&byloc, 'stat', \&bystat);

	if ($in{build} eq 'all')
	{
		genpage("All machines (sorted by $sorttext{$in{'sort'}})",
                        sort { &{$sortsub{$in{'sort'}}}; } @shosts);
	}
	else
	{
		genpage("Machines in building $in{build} " .
					"(sorted by $sorttext{$in{'sort'}})",
				sort { &{$sortsub{$in{'sort'}}}; }
					inbuilding($in{build}, @shosts));
	}
}

###############
# Subroutines #
###############

sub byloc
{
	$building{$a} cmp $building{$b}
	or
	$room{$a} cmp $room{$b}
	or
	byhostname();
	#$a cmp $b;
}

sub bystat
{
	$status{$a}[0] <=> $status{$b}[0]
	or
	byloc()
	or
	byhostname();
	#$a cmp $b;
}

sub genpage
{
	my $title = shift(@_);
	my $host;

	print <<EOF;
<HTML>
<HEAD>
<META HTTP-EQUIV="Refresh" CONTENT=$UPDATE>
<TITLE>$title</TITLE>
</HEAD>
<BODY>
<CENTER>
<H1>$title</H1>
<EM>This page generated at $now</EM><BR>
<EM>Status file last modified at $statmtime</EM>
</CENTER>

<HR>

<CENTER>
<H3>Statistics</H3>

<TABLE BORDER=2>
<TR>
	<TH>Status</TH>
	<TH>Count</TH>
</TR>
EOF

	my $code;
	my %status_count;
	foreach $host (@_)
	{
		my $code;

		foreach $code (@{$status{$host}})
		{
			$status_count{$code} = 0 unless (defined($status_count{$code}));
			$status_count{$code}++;
		}
	}
	foreach $code (sort { $a <=> $b } keys %status_count)
	{
		print "<TR><TD>$codestring{$code}</TD>";
		print "<TD>$status_count{$code}</TD></TR>\n";
	}

	print <<EOF;
</TABLE>
</CENTER>

<HR>

<CENTER>
<H3>Hosts</H3>
</CENTER>

<TABLE BORDER=2>
<TR>
	<TH>Hostname</TH>
	<TH>IP address</TH>
	<TH>Building</TH>
	<TH>Room</TH>
	<TH COLSPAN=2>Status</TH>
</TR>
EOF

	foreach $host (@_)
	{
		print "<TR><TD>$host</TD><TD>";
		if ($ip{$host})
		{
			print $ip{$host};
		}
		else
		{
			print "&nbsp;";
		}
		print "</TD><TD>";
		if ($building{$host})
		{
			print $building{$host};
		}
		else
		{
			print "&nbsp;";
		}
		print "</TD><TD>";
		if ($room{$host})
		{
			print $room{$host}
		}
		else
		{
			print "&nbsp;";
		}
		print "</TD><TD>";
		if ($status{$host}[0]%100 == 0)
		{
			print "<IMG SRC=\"images/greenball.gif\" ALT=\"\">";
		}
		elsif ($status{$host}[0]%100 <= 50)
		{
			print "<IMG SRC=\"images/redball.gif\" ALT=\"*\">";
		}
		else
		{
			print "<IMG SRC=\"images/yellowball.gif\" ALT=\"-\">";
		}
		print "</TD><TD>" . stattotext($host, 1) . "</TD></TR>\n";
	}

	# Print the footer
	print <<EOF;
</TABLE>

<HR>

<ADDRESS>jheiss\@hsc.hac.com</ADDRESS>

</BODY>
</HTML>
EOF

}

sub getbuilds
{
	my $host;
	my @builds = ();

	foreach $host (@_)
	{
		if (! inarray($building{$host}, @builds))
		{
			push(@builds, $building{$host});
		}
	}

	return @builds;
}

sub inbuilding
{
	my $host;
	my $build = shift(@_);
	my @inbuild = ();

	foreach $host (@_)
	{
		if ($building{$host} eq $build)
		{
			push(@inbuild, $host);
		}
	}

	return @inbuild;
}

