##############################################################################
# $Id: subnet.pl,v 1.2 1999/08/26 04:06:57 jheiss Exp $
##############################################################################
# Subroutines for checking various aspects of subnetting.
##############################################################################
# Copyright (C) 1999  Jason Heiss (jheiss@ofb.net)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##############################################################################
# Pre-RCS version control:
# created 10/16/97
# last modified 03/19/98
##############################################################################
# $Log: subnet.pl,v $
# Revision 1.2  1999/08/26 04:06:57  jheiss
# Added copyright and GNU license message.
#
# Revision 1.1  1999/08/09 03:16:15  jheiss
# Initial revision
#
##############################################################################

# These subroutines assumes that the subnet mask is 255.255.255.128.
# Oh well...
 
$DEBUGLEVEL = 0;

# Returns a value that represents which subnet the specified host is on.
sub subnet
{
	my $host = $_[0];

	my $ip;
	if (looks_like_ip($host))
	{
		$ip = $host;
	}
	else
	{
		($ip) = split(' ', `ypmatch $host hosts`);
	}
	my @ip_parts = split(/\./, $ip);

	print "Part 3 of $host:  $ip_parts[3]\n" if ($DEBUGLEVEL >= 2);

	if ($ip_parts[3] <= 127)
	{
		$low = 1;
	}
	else
	{
		$low = 0;
	}

	return join('.', $ip_parts[0], $ip_parts[1], $ip_parts[2], $low);
}

# Checks if two hosts are on the same subnet.
sub same_subnet
{
	my $hosta = $_[0];
	my $hostb = $_[1];

	print "$hosta: " . subnet($hosta) . "\n" if ($DEBUGLEVEL >= 1);
	print "$hostb: " . subnet($hostb) . "\n" if ($DEBUGLEVEL >= 1);

	if (subnet($hosta) eq subnet($hostb))
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

# This subroutine, given a host and list of other hosts (servers) will pick
# the first server in that list on the same subnet as the given host and
# return it.  Returns an empty string if there is no match.
sub pick_by_subnet
{
	my $host = shift(@_);
	
	foreach $server (@_)
	{
		print "Checking $server\n" if ($DEBUGLEVEL >= 2);
		foreach $inter (all_interfaces($server))
		{
			print "  Checking $inter\n" if ($DEBUGLEVEL >= 2);
			if (same_subnet($host, $inter))
			{
				#return $server;
				return $inter;
			}
		}
	}

	return '';
}

sub all_interfaces
{
	my $host = $_[0];
	my @all = ();

	open(NISHOSTS, "ypcat hosts.byaddr|");
	while(<NISHOSTS>)
	{
		chop;
		my ($ip, $name) = split;
		my ($base) = split(/\-/, $name);
		if ($base eq $host)
		{
			push(@all, $name);
		}
	}
	close(NISHOSTS);

	return @all;
}

# Subroutine for sorting a list of hosts by subnet
sub by_subnet
{
	subnet($a) cmp subnet($b)
	or
	$a cmp $b;
}

# Determines if a string looks like an IP address
sub looks_like_ip
{
	my $string = $_[0];

	my @parts = split(/\./, $string);

	if (scalar(@parts) == 4 &&
		$parts[0] =~ /^\d+$/ &&
		$parts[1] =~ /^\d+$/ &&
		$parts[2] =~ /^\d+$/ &&
		$parts[3] =~ /^\d+$/)
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

return 1;

